package jalview.gui;

import jalview.jbgui.GGroupEditor;
import jalview.datamodel.*;
import jalview.schemes.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;


public class GroupEditor extends GGroupEditor
{
  AlignViewport av;
  AlignmentPanel ap;
  Vector groups;
  Vector unassignedVector;
  Vector assignedVector=new Vector();
  boolean updating=false;
  int selectedGroupIndex = 0;
  Hashtable sequenceHash = new Hashtable();

  public GroupEditor(AlignViewport av, AlignmentPanel ap)
  {
    this.av = av;
    this.ap = ap;
    groups = av.getAlignment().getGroups();

    updateUnassigned();
    for (int i=0; i < ColourProperties.getColourSchemeNames().size(); i++)
       colourCombo.addItem(ColourProperties.getColourSchemeNames().elementAt(i));

    showGroup();
    updateGroupcombo();

    assignedList.addMouseListener( new MouseAdapter()
    {
      public void mouseClicked(MouseEvent e)
      {
        if (e.getClickCount() == 2)
        {    remove_actionPerformed(null);      }
      }
    });

    unassignedList.addMouseListener( new MouseAdapter()
    {
      public void mouseClicked(MouseEvent e)
      {
        if (e.getClickCount() == 2)
        { add_actionPerformed(null);    }
      }
    });
  }


  void updateGui()
  {
    unassignedList.setListData(unassignedVector);
    assignedList.setListData(assignedVector);
    updateGroupcombo();
    ap.RefreshPanels();
  }


  void updateGroupcombo()
  {
    updating=true;

    if(groupsCombo.getItemCount()>0)
      groupsCombo.removeAllItems();

    for (int i = 0; i < groups.size(); i++) {
      SequenceGroup sg = (SequenceGroup)groups.elementAt(i);
      groupsCombo.addItem( sg.getName() + " (" + sg.getSize() + " sequences)");
    }
    groupsCombo.setSelectedIndex(selectedGroupIndex);
    updating=false;
  }

  void updateUnassigned()
  {
    unassignedVector=new Vector();
    AlignmentI align = av.getAlignment();

    for (int i = 0; i < align.getHeight();i++)
    {
      sequenceHash.put( align.getSequenceAt(i).getDisplayId(), align.getSequenceAt(i) );
      boolean assigned = false;
      for(int j=0; j<groups.size(); j++)
      {
        SequenceGroup sg = (SequenceGroup)groups.elementAt(j);
        for(int k=0; k<sg.getSize(); k++)
        {
          if(sg.getSequenceAt(k).getDisplayId().equals( align.getSequenceAt(i).getDisplayId() ) )
          {
            assigned = true; break;
          }
        }

        if(assigned)
          break;
      }

      if(!assigned)
          unassignedVector.add(align.getSequenceAt(i).getDisplayId());

    }

    unassignedList.setListData(unassignedVector);
  }


  void showGroup()
  {
    assignedVector = new Vector();
    if(selectedGroupIndex!=-1)
    {
      SequenceGroup sg = (SequenceGroup) groups.elementAt(selectedGroupIndex);
      for (int j = 0; j < sg.getSize(); j++)
      {
        Sequence s = (Sequence) sg.getSequenceAt(j);
        assignedVector.addElement(s.getDisplayId());
      }
      int num = ColourProperties.indexOfClass(sg.cs);
      colourCombo.setSelectedIndex(num);

      boxesCheck.setSelected(sg.getDisplayBoxes());
      textCheck.setSelected(sg.getDisplayText());
      colourTextCheck.setSelected(sg.getColourText());
    }


    assignedList.setListData( assignedVector );
  }


  protected void groupsCombo_actionPerformed(ActionEvent e)
  {
    if( !updating )
    {
      selectedGroupIndex = groupsCombo.getSelectedIndex();
      showGroup();
    }
  }

  protected void groupName_actionPerformed(ActionEvent e)
  {
    String reply = JOptionPane.showInputDialog(this, "Enter new group name", "Edit group name", JOptionPane.QUESTION_MESSAGE);
    if(reply==null)
      return;

    SequenceGroup sg = (SequenceGroup)groups.elementAt(selectedGroupIndex);
    sg.setName(reply);
    updateGroupcombo();
  }

  protected void colourCombo_actionPerformed(ActionEvent e)
  {
     SequenceGroup sg = (SequenceGroup)groups.elementAt(selectedGroupIndex);

     sg.cs = ColourSchemeFactory.get( (String)colourCombo.getSelectedItem() );
     //ColourProperties.getColourScheme( colourCombo.getSelectedIndex() );
     ap.RefreshPanels();
  }

  protected void boxesCheck_actionPerformed(ActionEvent e)
  {
    SequenceGroup sg = (SequenceGroup)groups.elementAt(selectedGroupIndex);
    sg.setDisplayBoxes( boxesCheck.isSelected());
        ap.RefreshPanels();
  }

  protected void textCheck_actionPerformed(ActionEvent e)
  {
    SequenceGroup sg = (SequenceGroup)groups.elementAt(selectedGroupIndex);
    sg.setDisplayText( textCheck.isSelected() );
        ap.RefreshPanels();
  }

  protected void colourTextCheck_actionPerformed(ActionEvent e)
  {
    SequenceGroup sg = (SequenceGroup)groups.elementAt(selectedGroupIndex);
    sg.setColourText( colourTextCheck.isSelected());
        ap.RefreshPanels();
  }

  protected void addSelectedIds_actionPerformed(ActionEvent e)
  {
    if(selectedGroupIndex==-1)
      return;

    SequenceGroup sg = (SequenceGroup)groups.elementAt(selectedGroupIndex);
    Selection sel = av.getSelection();
    for(int i=0; i<sel.size(); i++)
    {
      int index = unassignedVector.indexOf( sel.sequenceAt(i).getDisplayId() );
      if(index!=-1)
      {
        sg.addSequence( (SequenceI) sequenceHash.get( unassignedVector.elementAt(index) ));
        assignedVector.addElement( unassignedVector.elementAt(index));
        unassignedVector.removeElementAt( index );
      }

    }
    updateGui();
  }

  protected void add_actionPerformed(ActionEvent e)
  {
    if(selectedGroupIndex==-1)
      return;

    int [] indices  = unassignedList.getSelectedIndices();
    if(indices.length<1)
      return;

    SequenceGroup sg = (SequenceGroup)groups.elementAt(selectedGroupIndex);

    for(int i=0; i<indices.length; i++)
    {
      assignedVector.addElement(unassignedVector.elementAt(indices[i]-i));
      sg.addSequence( (SequenceI)sequenceHash.get( unassignedVector.elementAt(indices[i]-i) ));
      unassignedVector.removeElementAt(indices[i]-i);
    }
    updateGui();
  }

  protected void addAll_actionPerformed(ActionEvent e)
  {
    if(selectedGroupIndex==-1)
      return;

    SequenceGroup sg = (SequenceGroup)groups.elementAt(selectedGroupIndex);
    Enumeration enum = unassignedVector.elements();
    while(enum.hasMoreElements())
    {
      String id = (String)enum.nextElement();
      assignedVector.addElement( id );
      sg.addSequence( (SequenceI)sequenceHash.get( id ));
    }

    unassignedVector.clear();
    updateGui();
  }

  protected void remove_actionPerformed(ActionEvent e)
  {
    if(selectedGroupIndex==-1)
      return;

    SequenceGroup sg = (SequenceGroup)groups.elementAt(selectedGroupIndex);

    int [] indices = assignedList.getSelectedIndices();
    if(indices.length<1)
      return;

    for(int i=0; i<indices.length; i++)
    {

      unassignedVector.addElement(assignedVector.elementAt(indices[i]-i));
      SequenceI removeSequence = (SequenceI)sequenceHash.get( assignedVector.elementAt(indices[i]-i) );
      av.getSelection().removeElement( removeSequence );
      sg.deleteSequence( removeSequence );
      assignedVector.removeElementAt(indices[i]-i);

    }

    updateGui();
  }

  protected void removeAll_actionPerformed(ActionEvent e)
  {
    if(selectedGroupIndex==-1)
      return;

    SequenceGroup sg = (SequenceGroup)groups.elementAt(selectedGroupIndex);
    Enumeration enum = assignedVector.elements();
    while(enum.hasMoreElements())
    {
      String id = (String)enum.nextElement();
      unassignedVector.addElement( id );
      SequenceI removeSequence = (SequenceI)sequenceHash.get( id );
      av.getSelection().removeElement( removeSequence );
      sg.deleteSequence( removeSequence );
    }

    assignedVector.clear();
    updateGui();
  }

  protected void addGroup_actionPerformed(ActionEvent e)
  {

   ColourSchemeI cs = ColourSchemeFactory.get( (String)colourCombo.getSelectedItem() );
    //   ColourProperties.getColourScheme( colourCombo.getSelectedIndex() );

   groups.add(
      new SequenceGroup("Group "+(groups.size()+1),
                        cs,
                        false,
                        boxesCheck.isSelected(),
                        textCheck.isSelected(),
                        colourTextCheck.isSelected(),
                        true) );

   updateGroupcombo();
   groupsCombo.setSelectedIndex( groupsCombo.getItemCount()-1 );
   showGroup();
  }

  protected void deleteGroup_actionPerformed(ActionEvent e)
  {
    removeAll_actionPerformed(null);
    groups.remove( selectedGroupIndex );
    selectedGroupIndex=-1;
    updateGroupcombo();
    groupsCombo.setSelectedIndex( groupsCombo.getItemCount()-1 );
    showGroup();
  }

}
